#' Connexion au serveur PostgreSQL
#'
#' @param db la database sur laquelle se connecter
#' @param user le profil utilisateur avec lequel se connecter : does ou dreal, suffixe utilise pour les variable d'environnement
#' @param user_id le nom utilisateur avec lequel se connecter tel que renseigné dans votre renviron
#' @param user_pwd le mot de passe avec lequel se connecter tel que renseigné dans votre renviron
#' @param server l'adresse ip du serveur PostgreSQL, laisser à NULL pour utiliser le variable d'environnement du .Renviron
#' @param ecoSQL mettre à TRUE pour switcher vers un driver adapté à l'usage de l'offre d'hébergement de base EcoSQL (Experimental)
#' @param ... eventuels autres arguments de la fonction d'ouverture de connexion `connect_to_db`
#' @return La fonction cree un connecteur pour se connecter à la base posgresql du serveur via le driver DBI "PostgreSQL" ou `RPostgres::Postgres()` si `ecoSQL = TRUE`.
#' @importFrom DBI dbDriver dbConnect
#' @importFrom RPostgres Postgres
#' @export
#'
#' @examples
#' \dontrun{
#' connect_to_db()
#' }
connect_to_db <- function(db = "datamart",
                          user = "does",
                          user_id = paste0("user_", user),
                          user_pwd = paste0("pwd_", user),
                          server = NULL,
                          ecoSQL = TRUE,
                          ...){

  if(ecoSQL) {
    drv <- RPostgres::Postgres()
  } else {
    drv <- DBI::dbDriver("PostgreSQL")
  }

  if(is.null(server)) {server <- Sys.getenv("server")}

  con <- DBI::dbConnect(drv,
                        dbname=db,
                        host=server,
                        port=Sys.getenv("port"),
                        user=Sys.getenv(user_id),
                        password=Sys.getenv(user_pwd),
                        ...)

  return(con)
}


#' Connexion au serveur PostgreSQL via ODBC connection string
#'
#' @param db la database sur laquelle se connecter
#' @param user le profil utilisateur avec lequel se connecter : does ou dreal
#' @param user_id le nom utilisateur avec lequel se connecter tel que renseigné dans votre renviron
#' @param user_pwd le mot de passe avec lequel se connecter tel que renseigné dans votre renviron
#' @param server l'adresse ip du serveur, laisser à NULL pour utiliser le variable d'environnement du .Renviron
#' @param driver choix du driver (actuellement seul PostgreSQL est accepté)
#' @return La fonction créé un connecteur pour se connecter à la base posgresql du serveur via le driver "PostgreSQL".
#' @return Un connecteur con
#' @importFrom DBI dbDriver dbConnect
#' @importFrom odbc odbc odbcListDrivers
#' @importFrom glue glue
#' @export
#'
#' @examples
#' \dontrun{
#' connect_to_dbi()
#' }
connect_to_dbi <- function(db="datamart",
                           user="does",
                           user_id=paste0("user_",user),
                           user_pwd = paste0("pwd_",user),
                           server = NULL,
                           driver = "PostgreSQL"){

  required_driver <- all(
    driver %in% c(NULL,"PostgreSQL") #liste a enrichir si on souhaite permettre d autre SGBD comme mariaDB...
  )
  if (isFALSE(required_driver)) {
    stop("The driver must be NULL or 'PostgreSQL'")
  }
  if(!is.null(driver) && driver == "PostgreSQL"){patterndriver <-"^Postgre.*UNICODE|Unicode.*"}
  else{patterndriver <- "^_"}#si driver NULL fail

  if(is.null(server)) {server <- Sys.getenv("server")}
  dbi_user <- Sys.getenv(user_id)
  dbi_pwd <- Sys.getenv(user_pwd)

  listdriver<-odbc::odbcListDrivers()
  drv_unicode_postgres <- listdriver$name[grepl(patterndriver, listdriver$name)][1]
  if(!is.na(drv_unicode_postgres)){
  #voir https://www.connectionstrings.com/
  con_chaine <- glue::glue("Driver={",drv_unicode_postgres,"};Server=$server$;Database=$db$;Uid=$dbi_user$;Pwd=$dbi_pwd$;Port=$Sys.getenv('port')$;",
                           .open = "$",
                           .close= "$")
  con <- DBI::dbConnect(odbc::odbc(),
                        .connection_string = con_chaine,
                        timeout = 10,
                        encoding = "UTF-8")
  }else{
    stop("alert: Pas de driver Postgresql pour ODBC")
  }
  # if(.Platform$OS.type == "windows") {
  #   DBI::dbExecute(con,"SET client_encoding TO \'windows-1252\'")
  #   }

  return(con)
}
