library(fs)
test_that("scoped_path_r works", {

  expect_error(scoped_path_r(scope="other"),
    "'arg' should be one of")

  Sys.setenv(R_ENVIRON_USER="~")
  res<-fs::path_expand("~")
  expect_equal(scoped_path_r("user", envvar = "R_ENVIRON_USER"), res)

  res<-fs::path(usethis::proj_get(), "README.md")
  expect_equal(scoped_path_r("project", "README.md"),res)

})

test_that("check_server_renviron works", {

  Sys.setenv(R_ENVIRON_USER="~/test-toto")
  expect_message(check_server_renviron(),
                 paste0("x .Renviron : le fichier ",scoped_path_r("user", ".Renviron", envvar = "R_ENVIRON_USER")," n'existe pas"))

  Sys.setenv(R_ENVIRON_USER="~")
  if (.Platform$OS.type != "windows" & !file.exists(scoped_path_r("user", ".Renviron", envvar = "R_ENVIRON_USER"))) {
    file.create(scoped_path_r("user", ".Renviron", envvar = "R_ENVIRON_USER"))
  }

  Sys.setenv(server="")
  Sys.setenv(port="")
  Sys.setenv(user_does="")
  Sys.setenv(pwd_does="")
  expect_message(check_server_renviron(), paste0("i v\u00e9rification de la configuration des variables syst\u00e8me \u00e0 l'adresse ",scoped_path_r("user", ".Renviron", envvar = "R_ENVIRON_USER"),"."))
  expect_message(check_server_renviron(), "* .Renviron : la variable server n'est pas d\u00e9finie.")
  expect_message(check_server_renviron(), "* .Renviron : la variable port n'est pas d\u00e9finie.")
  expect_message(check_server_renviron(), "* .Renviron : la variable user_does n'est pas d\u00e9finie.")
  expect_message(check_server_renviron(), "* .Renviron : la variable pwd_does n'est pas d\u00e9finie.")

  Sys.setenv(server="127.0.0.1")
  Sys.setenv(port="5432")
  Sys.setenv(user_does="runner")
  Sys.setenv(pwd_does=Sys.getenv("BDD_RUNNER_PWD"))

  expect_message(check_server_renviron(), paste0("v .Renviron : la variable server  est d\u00e9finie avec la valeur ",Sys.getenv('server')))
  expect_message(check_server_renviron(), paste0("v .Renviron : la variable port est d\u00e9finie avec la valeur ",Sys.getenv('port')))
  expect_message(check_server_renviron(), paste0("v .Renviron : la variable user_does est d\u00e9finie avec la valeur ",Sys.getenv('user_does')))
  expect_message(check_server_renviron(), paste0("v .Renviron : la variable pwd_does est d\u00e9finie avec la valeur ",Sys.getenv('pwd_does')))
})
