library(DBI)
library(magrittr)
library(tibble)
library(lubridate)
library(dplyr)
library(glue)

test_that("commenter_champs works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    expect_error(commenter_champs(var = "Species", table = "test_iris", schema = "public", db = "test_datalibaba", user = "test"),
                 "L\'argument comment n\'est pas renseign\u00e9. ")
    expect_error(commenter_champs(comment = "Commentlibaba !", table = "test_iris", schema = "public", db = "test_datalibaba", user = "test"),
                 "L\'argument var n\'est pas renseign\u00e9. ")
    expect_error(commenter_champs(comment = "Commentlibaba !", var = "Species", table = "test_iris", db = "test_datalibaba", user = "test"),
                 "L\'argument schema n\'est pas renseign\u00e9. ")
    expect_error(commenter_champs(comment = "Commentlibaba !", var = "Species", schema = "public", db = "test_datalibaba", user = "test"),
                 "L\'argument table n\'est pas renseign\u00e9. ")
    expect_error(commenter_champs(comment = "Commentlibaba !", var = "Species", table = "test_titi", schema = "public", db = "test_datalibaba", user = "test"),
                 "Il n\'y a pas de table test_titi dans le schema public. ")
    expect_error(commenter_champs(comment = "Commentlibaba !", var = "titi", table = "test_iris", schema = "public", db = "test_datalibaba", user = "test"),
                 "Il n\'y a pas de champ titi dans la table test_iris. ")

    expect_null(commenter_champs(comment = "Commentlibaba !", var = "Species", table = "test_iris", schema = "public", db = "test_datalibaba", user = "test"))

  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("post_dico_attr works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    dico_iris <- data.frame(VAR = names(iris), VAR_LIB = paste("libell\u00e9 de", names(iris), " cc"))
    expect_error(post_dico_attr(dico = c(1), table = "test_iris", schema = "public", db = "test_datalibaba", user = "test"),
                 "L\'argument dico ne correpond pas \u00e0 un dataframe. ")
    expect_error(post_dico_attr(dico = data.frame(), table = "test_iris", schema = "public", db = "test_datalibaba", user = "test"),
                 "L\'argument dico est vide. ")
    expect_error(post_dico_attr(dico = dico_iris %>% mutate(otr="detrop"), table = "test_iris", schema = "public", db = "test_datalibaba", user = "test"),
                 "L\'argument dico doit comprendre 2 champs : le 1er pour le nom des variables, le second pour leur explicitation. ")
    expect_error(post_dico_attr(dico = dico_iris, table = "test_iris", db = "test_datalibaba", user = "test"),
                 "L\'argument schema n\'est pas renseign\u00e9. ")
    expect_error(post_dico_attr(dico = dico_iris, schema = "public", db = "test_datalibaba", user = "test"),
                 "L\'argument table n\'est pas renseign\u00e9. ")
    expect_error(post_dico_attr(dico = dico_iris, table = "titi_test", schema = "public", db = "test_datalibaba", user = "test"),
                 "Il n\'y a pas de table titi_test dans le schema public. ")
    expect_error(post_dico_attr(dico = dico_iris, table = "nc", schema = "public", db = "test_datalibaba", user = "test"),
                 "Les champs list\u00e9s dans 'dico' ne se retrouvent pas tous dans la table nc. ")

    expect_null(post_dico_attr(dico = dico_iris, table = "retest_iris", schema = "public", db = "test_datalibaba", user = "test"))

  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("get_cols_comment works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    expect_error(get_cols_comment(table = "test_iris", db = "test_datalibaba", user = "test", server = Sys.getenv("server")),
                 "L\'argument schema n\'est pas renseign\u00e9. ")
    expect_error(get_cols_comment(schema = "public", db = "test_datalibaba", user = "test", server = Sys.getenv("server")),
                 "L\'argument table n\'est pas renseign\u00e9. ")
    expect_error(get_cols_comment(table = "titi_test", schema = "public", db = "test_datalibaba", user = "test", server = Sys.getenv("server")),
                 "Il n\'y a pas de table titi_test dans le schema public. ")

    expect_snapshot(get_cols_comment(table = "test_iris", schema = "public", db = "test_datalibaba", user = "test", server = Sys.getenv("server")))

  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("get_table_comments works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    expect_snapshot(get_table_comments(table = "retest_iris", schema = "public", db = "test_datalibaba", user = "test", server = Sys.getenv("server")))

  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("get_table_comment works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    expect_error(get_table_comment(table = "test_iris", db = "test_datalibaba", user = "test", server = Sys.getenv("server")),
                 "L\'argument schema n\'est pas renseign\u00e9. ")
    expect_error(get_table_comment(schema = "public", db = "test_datalibaba", user = "test", server = Sys.getenv("server")),
                 "L\'argument table n\'est pas renseign\u00e9. ")
    expect_error(get_table_comment(table = "titi_test", schema = "public", db = "test_datalibaba", user = "test", server = Sys.getenv("server")),
                 "Il n\'y a pas de table titi_test dans le schema public. ")

    expect_snapshot(get_table_comment(table = "retest_iris", schema = "public", db = "test_datalibaba", user = "test", server = Sys.getenv("server")))


  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("transferer_table_comments works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    expect_null(transferer_table_comments(table_sce = "retest_iris", schema_sce = "public", db_sce = "test_datalibaba", table_dest = "test_iris", schema_dest = "public", db_dest = "test_datalibaba", user = "test", server = Sys.getenv("server")))

  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("exporter_table_comments works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    mon_dossier_temp_aleatoire <- tempfile(pattern = "test_exporter_table_comments")
    dir.create(mon_dossier_temp_aleatoire)
    fichier_temp <- file.path(mon_dossier_temp_aleatoire, "comment_iris.csv")

    expect_message(exporter_table_comments(table = "retest_iris", schema = "public", db  = "test_datalibaba", user = "test", server = Sys.getenv("server"), fichier = fichier_temp, import = TRUE),
                   "Export du dictionnaire des variables csv")

    # On vérifie que le fichier a bien été écrit en sortie
    expect_true(file.exists(fichier_temp))
    # On vérifie que la taille du fichier est non nulle
    expect_true(file.info(fichier_temp)$size > 0)

    unlink(mon_dossier_temp_aleatoire, recursive = TRUE)


  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})
